package main

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log"
	"math/rand"
	"net/http"
	"os"

	"github.com/gorilla/mux"
)

type Arrows struct {
	Arrows []Arrow `json:"arrows"`
}

type Arrow struct {
	Direction Vector `json:"direction"`
	Base      Vector `json:"base"`
}

type Vector struct {
	X float64 `json:"x"`
	Y float64 `json:"y"`
	Z float64 `json:"z"`
}

func main() {
	router := mux.NewRouter()
	router.HandleFunc("/arrows", getArrows).Methods("GET")
	router.HandleFunc("/arrows", addCorsHeader).Methods("OPTIONS")
	log.Fatal(http.ListenAndServe(":8081", router))
}

func parseFile(path string) Arrows {
	jsonFile, error := os.Open(path)
	if error != nil {
		fmt.Println(error)
	}
	defer jsonFile.Close()
	byteValue, _ := ioutil.ReadAll(jsonFile)
	var arrows Arrows
	json.Unmarshal(byteValue, &arrows)
	randomize(arrows)
	return arrows
}

func getArrows(w http.ResponseWriter, r *http.Request) {
	addCorsHeader(w, r)
	w.Header().Set("Content-Type", "application/json")
	json.NewEncoder(w).Encode(parseFile("input/data.json"))
}

func addCorsHeader(w http.ResponseWriter, r *http.Request) {
	headers := w.Header()
	headers.Add("Access-Control-Allow-Origin", "*")
	headers.Add("Vary", "Origin")
	headers.Add("Vary", "Access-Control-Request-Method")
	headers.Add("Vary", "Access-Control-Request-Headers")
	headers.Add("Access-Control-Allow-Headers", "Content-Type, Origin, Accept, token")
	headers.Add("Access-Control-Allow-Methods", "GET, POST, OPTIONS")
}

func randomize(arrows Arrows) {
	for i := 0; i < len(arrows.Arrows); i++ {
		arrows.Arrows[i].Direction.X += rand.Float64() - 0.5
		arrows.Arrows[i].Direction.Y += rand.Float64() - 0.5
		arrows.Arrows[i].Direction.Z += rand.Float64() - 0.5
		arrows.Arrows[i].Base.X += rand.Float64() - 0.5
		arrows.Arrows[i].Base.Y += rand.Float64() - 0.5
		arrows.Arrows[i].Base.Z += rand.Float64() - 0.5
	}
}
